@@ -32,9 +32,7 @@
 #include "gs-job.h"
 #include "gs-grab.h"
 
-#include "fade.h"
-
-#define  FADE_SECONDS 2
+#include "gs-fade.h"
 
 static void gs_manager_class_init (GSManagerClass *klass);
 static void gs_manager_init       (GSManager      *manager);
@@ -93,6 +91,8 @@ enum {
         PROP_THROTTLE_ENABLED,
 };
 
+#define FADE_TIMEOUT 1000
+
 static GObjectClass *parent_class = NULL;
 static guint         signals [LAST_SIGNAL] = { 0, };
 
@@ -944,7 +944,15 @@ gs_manager_activate (GSManager *manager)
         /* fade to black and show windows */
         do_fade = TRUE;
         if (do_fade) {
-                fade_screens_and_show (FADE_SECONDS, 20, TRUE, manager->priv->windows);
+                GSFade *fade;
+
+                fade = gs_fade_new ();
+                gs_fade_set_timeout (fade, FADE_TIMEOUT);
+                gs_fade_now (fade);
+                show_windows (manager->priv->windows);
+                g_usleep (500000);
+                gs_fade_reset (fade);
+                g_object_unref (fade);
         } else {
                 show_windows (manager->priv->windows);
         }
