@@ -43,7 +43,7 @@
 #include <gdk/gdkx.h>
 
 #include "gnome-screensaver.h"
-#include "gs-watcher-x11.h"
+#include "gs-watcher.h"
 #include "gs-marshal.h"
 
 static void     gs_watcher_class_init (GSWatcherClass *klass);
@@ -83,9 +83,13 @@ struct GSWatcherPrivate
         guint           use_sgi_saver_extension : 1;
         guint           use_mit_saver_extension : 1;
 
+        guint           notice_timeout;
+
         /* state */
         guint            active : 1;
 
+        guint            notice_sent : 1;
+
         PointerPosition *poll_position;
 
         time_t          last_wall_clock_time;
@@ -114,12 +118,15 @@ struct GSWatcherPrivate
 
 enum {
         IDLE,
+        IDLE_NOTICE,
+        NOTICE_CANCELLED,
         LAST_SIGNAL
 };
 
 enum {
         PROP_0,
-        PROP_TIMEOUT
+        PROP_TIMEOUT,
+        PROP_DEBUG
 };
 
 static GObjectClass *parent_class = NULL;
@@ -157,6 +164,17 @@ gs_watcher_set_timeout (GSWatcher  *watcher,
         }
 }
 
+static void
+_gs_watcher_set_debug (GSWatcher  *watcher,
+                       gboolean    enabled)
+{
+        g_return_if_fail (GS_IS_WATCHER (watcher));
+
+        if (watcher->priv->debug != enabled) {
+                watcher->priv->debug = enabled;
+        }
+}
+
 static void
 gs_watcher_set_property (GObject            *object,
                          guint               prop_id,
@@ -171,6 +189,9 @@ gs_watcher_set_property (GObject            *object,
         case PROP_TIMEOUT:
                 gs_watcher_set_timeout (self, g_value_get_uint (value));
                 break;
+        case PROP_DEBUG:
+                _gs_watcher_set_debug (self, g_value_get_boolean (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -191,6 +212,9 @@ gs_watcher_get_property (GObject            *object,
         case PROP_TIMEOUT:
                 g_value_set_uint (value, self->priv->timeout);
                 break;
+        case PROP_DEBUG:
+                g_value_set_boolean (value, self->priv->debug);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -218,6 +242,27 @@ gs_watcher_class_init (GSWatcherClass *klass)
                               gs_marshal_BOOLEAN__INT,
                               G_TYPE_BOOLEAN,
                               1, G_TYPE_INT);
+        signals [IDLE_NOTICE] =
+                g_signal_new ("idle_notice",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GSWatcherClass, idle_notice),
+                              NULL,
+                              NULL,
+                              gs_marshal_BOOLEAN__INT,
+                              G_TYPE_BOOLEAN,
+                              1, G_TYPE_INT);
+
+        signals [NOTICE_CANCELLED] =
+                g_signal_new ("notice_cancelled",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GSWatcherClass, notice_cancelled),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0, G_TYPE_NONE);
 
         g_object_class_install_property (object_class,
                                          PROP_TIMEOUT,
@@ -228,6 +273,13 @@ gs_watcher_class_init (GSWatcherClass *klass)
                                                             G_MAXUINT,
                                                             600000,
                                                             G_PARAM_READWRITE));
+        g_object_class_install_property (object_class,
+                                         PROP_DEBUG,
+                                         g_param_spec_boolean ("debug",
+                                                               NULL,
+                                                               NULL,
+                                                               FALSE,
+                                                               G_PARAM_READWRITE));
 
         g_type_class_add_private (klass, sizeof (GSWatcherPrivate));
 }
@@ -489,10 +541,25 @@ reset_timers (GSWatcher *watcher)
 static void
 _gs_watcher_notice_activity (GSWatcher *watcher)
 {
+        if (! watcher->priv->active) {
+                g_warning ("Noticed activity but watcher is inactive");
+                return;
+        }
+
         if (watcher->priv->debug) {
                 g_message ("Activity detected: resetting timers");
         }
 
+        /* if an idle notice was sent, cancel it */
+        if (watcher->priv->notice_sent) {
+                if (watcher->priv->debug) {
+                        g_message ("Cancelling idle notice");
+                }
+
+                g_signal_emit (watcher, signals [NOTICE_CANCELLED], 0);
+                watcher->priv->notice_sent = FALSE;
+        }
+
         reset_timers (watcher);
 }
 
@@ -729,11 +796,14 @@ gs_watcher_init (GSWatcher *watcher)
         watcher->priv->enabled = TRUE;
         watcher->priv->active = FALSE;
         watcher->priv->timeout = 600000;
-        watcher->priv->pointer_timeout = 5000;
+        watcher->priv->pointer_timeout = 2000;
 
         watcher->priv->verbose = FALSE;
         watcher->priv->debug = FALSE;
 
+        /* time before idle signal to send notice signal */
+        watcher->priv->notice_timeout = 10000;
+
         initialize_server_extensions (watcher);
 
         add_watchdog_timer (watcher, 600000);
@@ -1068,17 +1138,18 @@ maybe_send_signal (GSWatcher *watcher)
 {
         gboolean polling_for_idleness = TRUE;
         int      idle;
-        gboolean do_signal = FALSE;
+        gboolean do_idle_signal = FALSE;
+        gboolean do_notice_signal = FALSE;
 
         idle = 1000 * (time (NULL) - watcher->priv->last_activity_time);
 
         if (idle >= watcher->priv->timeout) {
                 /* Look, we've been idle long enough.  We're done. */
-                do_signal = TRUE;
+                do_idle_signal = TRUE;
         } else if (watcher->priv->emergency_lock) {
                 /* Oops, the wall clock has jumped far into the future, so
                    we need to lock down in a hurry! */
-                do_signal = TRUE;
+                do_idle_signal = TRUE;
         } else {
                 /* The event went off, but it turns out that the user has not
                    yet been idle for long enough.  So re-signal the event.
@@ -1086,22 +1157,48 @@ maybe_send_signal (GSWatcher *watcher)
                    user has been idle for 2 minutes, then set this timer to
                    go off in 3 minutes.
                 */
-                if (polling_for_idleness)
-                        schedule_wakeup_event (watcher, watcher->priv->timeout - idle, watcher->priv->debug);
-                do_signal = FALSE;
+
+                if (polling_for_idleness) {
+                        guint timeout;
+
+                        timeout = watcher->priv->timeout - idle;
+
+                        if (timeout >= watcher->priv->notice_timeout) {
+                                do_notice_signal = TRUE;
+                        }
+
+                        schedule_wakeup_event (watcher, timeout, watcher->priv->debug);
+                }
+
+                do_idle_signal = FALSE;
         }
 
-        if (do_signal) {
+        if (do_notice_signal
+            && ! watcher->priv->notice_sent) {
+		gboolean res = FALSE;
+
+                if (watcher->priv->debug) {
+                        g_message ("Sending idle notice");
+                }
+
+                watcher->priv->notice_sent = TRUE;
+                g_signal_emit (watcher, signals [IDLE_NOTICE], 0, 0, &res);
+        }
+
+        if (do_idle_signal) {
 		gboolean res = FALSE;
 
                 g_signal_emit (watcher, signals [IDLE], 0, 0, &res);
 
+                watcher->priv->notice_sent = FALSE;
+
                 /* if the event wasn't handled then schedule another timer */
                 if (! res) {
                         g_warning ("Idle signal was not handled, restarting watcher");
 
-                        if (polling_for_idleness)
+                        if (polling_for_idleness) {
                                 schedule_wakeup_event (watcher, watcher->priv->timeout, watcher->priv->debug);
+                        }
                 }
         }
 }
@@ -1124,14 +1221,23 @@ schedule_wakeup_event (GSWatcher *watcher,
                        int        when,
                        gboolean   verbose)
 {
+        guint timeout;
+
         if (watcher->priv->timer_id) {
                 if (verbose)
                         g_message ("idle_timer already running");
                 return;
         }
+        
+        timeout = when;
+
+        /* Wake up before idle so we can send a notice signal */
+        if (timeout > watcher->priv->notice_timeout) {
+                timeout -= watcher->priv->notice_timeout;
+        }
 
         /* Wake up periodically to ask the server if we are idle. */
-        add_idle_timer (watcher, when);
+        add_idle_timer (watcher, timeout);
 }
 
 /* An unfortunate situation is this: the saver is not active, because the
