@@ -38,6 +38,9 @@ static void gs_window_class_init (GSWindowClass *klass);
 static void gs_window_init       (GSWindow      *window);
 static void gs_window_finalize   (GObject       *object);
 
+static gboolean popup_dialog_idle (GSWindow *window);
+static gboolean gs_window_request_unlock_idle (GSWindow *window);
+
 enum {
         DIALOG_RESPONSE_CANCEL,
         DIALOG_RESPONSE_OK
@@ -82,7 +85,7 @@ struct GSWindowPrivate
 };
 
 enum {
-        UNBLANKED,
+        DEACTIVATED,
         DIALOG_UP,
         DIALOG_DOWN,
         LAST_SIGNAL
@@ -100,7 +103,7 @@ enum {
 static GObjectClass   *parent_class = NULL;
 static guint           signals [LAST_SIGNAL] = { 0, };
 
-G_DEFINE_TYPE (GSWindow, gs_window, GTK_TYPE_WINDOW);
+G_DEFINE_TYPE (GSWindow, gs_window, GTK_TYPE_WINDOW)
 
 static void
 set_invisible_cursor (GdkWindow *window,
@@ -220,6 +223,17 @@ gs_window_move_resize_window (GSWindow *window,
                                    window->priv->geometry.height);
 }
 
+static void
+gs_window_real_unrealize (GtkWidget *widget)
+{
+        g_signal_handlers_disconnect_by_func (gtk_window_get_screen (GTK_WINDOW (widget)),
+                                              screen_size_changed,
+                                              widget);
+
+        if (GTK_WIDGET_CLASS (parent_class)->unrealize)
+                GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+}
+
 static void
 gs_window_real_realize (GtkWidget *widget)
 {
@@ -247,6 +261,125 @@ watchdog_timer (GSWindow *window)
         return TRUE;
 }
 
+static void
+remove_watchdog_timer (GSWindow *window)
+{
+        if (window->priv->watchdog_timer_id != 0) {
+                g_source_remove (window->priv->watchdog_timer_id);
+                window->priv->watchdog_timer_id = 0;
+        }
+}
+
+static void
+add_watchdog_timer (GSWindow *window,
+                    glong     timeout)
+{
+        window->priv->watchdog_timer_id = g_timeout_add (timeout,
+                                                         (GSourceFunc)watchdog_timer,
+                                                         window);
+}
+
+static void
+remove_popup_dialog_idle (GSWindow *window)
+{
+        if (window->priv->popup_dialog_idle_id != 0) {
+                g_source_remove (window->priv->popup_dialog_idle_id);
+                window->priv->popup_dialog_idle_id = 0;
+        }
+}
+
+static void
+add_popup_dialog_idle (GSWindow *window)
+{
+        window->priv->popup_dialog_idle_id = g_idle_add ((GSourceFunc)popup_dialog_idle, window);
+}
+
+static void
+remove_request_unlock_idle (GSWindow *window)
+{
+        if (window->priv->request_unlock_idle_id != 0) {
+                g_source_remove (window->priv->request_unlock_idle_id);
+                window->priv->request_unlock_idle_id = 0;
+        }
+}
+
+static void
+add_request_unlock_idle (GSWindow *window)
+{
+        window->priv->request_unlock_idle_id = g_idle_add ((GSourceFunc)gs_window_request_unlock_idle, window);
+}
+
+static gboolean
+emit_deactivated_idle (GSWindow *window)
+{
+        g_signal_emit (window, signals [DEACTIVATED], 0);
+
+        return FALSE;
+}
+
+static void
+add_emit_deactivated_idle (GSWindow *window)
+{
+        g_idle_add ((GSourceFunc)emit_deactivated_idle, window);
+}
+
+static void
+gs_window_raise (GSWindow *window)
+{
+        GdkWindow *win;
+
+        g_return_if_fail (GS_IS_WINDOW (window));
+
+        win = GTK_WIDGET (window)->window;
+
+        gdk_window_raise (win);
+}
+
+static void
+gs_window_xevent (GSWindow  *window,
+                  GdkXEvent *xevent)
+{
+        XEvent *ev;
+
+        ev = xevent;
+
+        /* MapNotify is used to tell us when new windows are mapped.
+           ConfigureNofify is used to tell us when windows are raised. */
+        switch (ev->xany.type) {
+        case MapNotify:
+                gs_window_raise (window);
+                break;
+        case ConfigureNotify:
+                gs_window_raise (window);
+                break;
+        default:
+                break;
+        }
+
+}
+
+static GdkFilterReturn
+xevent_filter (GdkXEvent *xevent,
+               GdkEvent  *event,
+               GSWindow  *window)
+{
+        gs_window_xevent (window, xevent);
+
+        return GDK_FILTER_CONTINUE;
+}
+
+static void
+select_popup_events (void)
+{
+        XWindowAttributes attr;
+        unsigned long     events;
+
+        XGetWindowAttributes (GDK_DISPLAY (), GDK_ROOT_WINDOW(), &attr);
+
+        events = SubstructureNotifyMask | attr.your_event_mask;
+        XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW(), events);
+}
+
 static void
 gs_window_real_show (GtkWidget *widget)
 {
@@ -262,11 +395,11 @@ gs_window_real_show (GtkWidget *widget)
                 g_timer_destroy (window->priv->timer);
         window->priv->timer = g_timer_new ();
 
-        if (window->priv->watchdog_timer_id != 0)
-                g_source_remove (window->priv->watchdog_timer_id);
-        window->priv->watchdog_timer_id = g_timeout_add (30000,
-                                                         (GSourceFunc)watchdog_timer,
-                                                         window);
+        remove_watchdog_timer (window);
+        add_watchdog_timer (window, 30000);
+
+        select_popup_events ();
+        gdk_window_add_filter (NULL, (GdkFilterFunc)xevent_filter, window);
 }
 
 void
@@ -284,8 +417,9 @@ gs_window_real_hide (GtkWidget *widget)
 
         window = GS_WINDOW (widget);
 
-        if (window->priv->watchdog_timer_id != 0)
-                g_source_remove (window->priv->watchdog_timer_id);
+        gdk_window_remove_filter (NULL, (GdkFilterFunc)xevent_filter, window);
+
+        remove_watchdog_timer (window);
 
         if (GTK_WIDGET_CLASS (parent_class)->hide)
                 GTK_WIDGET_CLASS (parent_class)->hide (widget);
@@ -307,14 +441,6 @@ gs_window_get_gdk_window (GSWindow *window)
         return GTK_WIDGET (window)->window;
 }
 
-static gboolean
-emit_unblanked_idle (GSWindow *window)
-{
-        g_signal_emit (window, signals [UNBLANKED], 0);
-
-        return FALSE;
-}
-
 static gboolean
 spawn_on_window (GSWindow *window,
                  char     *command,
@@ -327,6 +453,7 @@ spawn_on_window (GSWindow *window,
         char      **argv;
         char       *str;
         GPtrArray  *env;
+        GError     *error;
         gboolean    result;
         GIOChannel *channel;
         int         standard_output;
@@ -334,8 +461,12 @@ spawn_on_window (GSWindow *window,
         int         id;
         int         i;
 
-        if (!g_shell_parse_argv (command, &argc, &argv, NULL))
+        error = NULL;
+        if (! g_shell_parse_argv (command, &argc, &argv, &error)) {
+                g_warning ("Could not parse command: %s", error->message);
+                g_error_free (error);
                 return FALSE;
+        }
 
         env = g_ptr_array_new ();
 
@@ -365,6 +496,7 @@ spawn_on_window (GSWindow *window,
                                                        g_getenv ("LANGUAGE")));
         g_ptr_array_add (env, NULL);
 
+        error = NULL;
         result = gdk_spawn_on_screen_with_pipes (GTK_WINDOW (window)->screen,
                                                  g_get_home_dir (),
                                                  argv,
@@ -376,13 +508,15 @@ spawn_on_window (GSWindow *window,
                                                  NULL,
                                                  &standard_output,
                                                  NULL,
-                                                 NULL);
+                                                 &error);
 
         for (i = 0; i < env->len; i++)
                 g_free (g_ptr_array_index (env, i));
         g_ptr_array_free (env, TRUE);
 
         if (! result) {
+                g_warning ("Could not start command '%s': %s", command, error->message);
+                g_error_free (error);
                 g_strfreev (argv);
                 return FALSE;
         }
@@ -473,6 +607,11 @@ static void
 socket_destroyed (GtkWidget *widget,
                   GSWindow  *window)
 {
+        g_signal_handlers_disconnect_by_func (window->priv->socket, socket_show, window);
+        g_signal_handlers_disconnect_by_func (window->priv->socket, socket_destroyed, window);
+        g_signal_handlers_disconnect_by_func (window->priv->socket, plug_added, window);
+        g_signal_handlers_disconnect_by_func (window->priv->socket, plug_removed, window);
+
         window->priv->socket = NULL;
 }
 
@@ -528,10 +667,10 @@ gs_window_dialog_finish (GSWindow *window)
                 int exit_status;
                         
                 exit_status = wait_on_child (window->priv->pid);
-        }
 
-        g_spawn_close_pid (window->priv->pid);
-        window->priv->pid = 0;
+                g_spawn_close_pid (window->priv->pid);
+                window->priv->pid = 0;
+        }
 
         /* remove events for the case were we failed to show socket */
         remove_key_events (window);
@@ -629,7 +768,7 @@ command_watch (GIOChannel   *source,
                 gs_window_dialog_finish (window);
 
                 if (window->priv->dialog_response == DIALOG_RESPONSE_OK) {
-                        g_idle_add ((GSourceFunc)emit_unblanked_idle, window);
+                        add_emit_deactivated_idle (window);
                 }
 
                 gs_window_clear (window);
@@ -641,6 +780,7 @@ command_watch (GIOChannel   *source,
                 window->priv->last_y = -1;
 
                 window->priv->watch_id = 0;
+
                 return FALSE;
         }
 
@@ -687,6 +827,7 @@ popup_dialog_idle (GSWindow *window)
                 command = g_string_append (command, " --enable-logout");
                 g_string_append_printf (command, " --logout-command='%s'", window->priv->logout_command);
         }
+
         if (is_user_switch_enabled (window)) {
                 command = g_string_append (command, " --enable-switch");
         }
@@ -700,7 +841,7 @@ popup_dialog_idle (GSWindow *window)
                                   (GIOFunc)command_watch,
                                   window,
                                   &window->priv->watch_id);
-        if (!result)
+        if (! result)
                 g_warning ("Could not start command: %s", command->str);
 
         g_string_free (command, TRUE);
@@ -719,12 +860,13 @@ gs_window_request_unlock (GSWindow *window)
                 return;
 
         if (! window->priv->lock_enabled) {
-                g_idle_add ((GSourceFunc)emit_unblanked_idle, window);
+                add_emit_deactivated_idle (window);
+
                 return;
         }
 
         if (window->priv->popup_dialog_idle_id == 0) {
-                window->priv->popup_dialog_idle_id = g_idle_add ((GSourceFunc)popup_dialog_idle, window);
+                add_popup_dialog_idle (window);
         }
 
         g_signal_emit (window, signals [DIALOG_UP], 0);
@@ -931,7 +1073,7 @@ gs_window_real_key_press_event (GtkWidget   *widget,
         /* if we don't already have a socket then request an unlock */
         if (! GS_WINDOW (widget)->priv->socket) {
                 if (GS_WINDOW (widget)->priv->request_unlock_idle_id == 0) {
-                        GS_WINDOW (widget)->priv->request_unlock_idle_id = g_idle_add ((GSourceFunc)gs_window_request_unlock_idle, widget);
+                        add_request_unlock_idle (GS_WINDOW (widget));
                 }
 
                 catch_events = TRUE;
@@ -978,7 +1120,7 @@ gs_window_real_motion_notify_event (GtkWidget      *widget,
                 /* if we don't already have a socket then request an unlock */
                 if (! window->priv->socket
                     && (window->priv->request_unlock_idle_id == 0)) {
-                        window->priv->request_unlock_idle_id = g_idle_add ((GSourceFunc)gs_window_request_unlock_idle, widget);
+                        add_request_unlock_idle (window);
                 }
                 window->priv->last_x = -1;
                 window->priv->last_y = -1;
@@ -1039,17 +1181,18 @@ gs_window_class_init (GSWindowClass *klass)
         widget_class->show                = gs_window_real_show;
         widget_class->hide                = gs_window_real_hide;
         widget_class->realize             = gs_window_real_realize;
+        widget_class->unrealize           = gs_window_real_unrealize;
         widget_class->key_press_event     = gs_window_real_key_press_event;
         widget_class->motion_notify_event = gs_window_real_motion_notify_event;
         widget_class->size_request        = gs_window_real_size_request;
 
         g_type_class_add_private (klass, sizeof (GSWindowPrivate));
 
-        signals [UNBLANKED] =
-                g_signal_new ("unblanked",
+        signals [DEACTIVATED] =
+                g_signal_new ("deactivated",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GSWindowClass, unblanked),
+                              G_STRUCT_OFFSET (GSWindowClass, deactivated),
                               NULL,
                               NULL,
                               g_cclosure_marshal_VOID__VOID,
@@ -1153,6 +1296,15 @@ gs_window_init (GSWindow *window)
                                | GDK_LEAVE_NOTIFY_MASK);
 }
 
+static void
+remove_command_watch (GSWindow *window)
+{
+        if (window->priv->watch_id != 0) {
+                g_source_remove (window->priv->watch_id);
+                window->priv->watch_id = 0;
+        }
+}
+
 static void
 gs_window_finalize (GObject *object)
 {
@@ -1167,27 +1319,24 @@ gs_window_finalize (GObject *object)
 
         g_free (window->priv->logout_command);
 
-        if (window->priv->watchdog_timer_id != 0) {
-                g_source_remove (window->priv->watchdog_timer_id);
-                window->priv->watchdog_timer_id = 0;
-        }
-
-        if (window->priv->request_unlock_idle_id != 0) {
-                g_source_remove (window->priv->request_unlock_idle_id);
-                window->priv->request_unlock_idle_id = 0;
-        }
-
-        if (window->priv->popup_dialog_idle_id != 0) {
-                g_source_remove (window->priv->popup_dialog_idle_id);
-                window->priv->popup_dialog_idle_id = 0;
-        }
+        remove_watchdog_timer (window);
+        remove_request_unlock_idle (window);
+        remove_popup_dialog_idle (window);
 
         if (window->priv->timer)
                 g_timer_destroy (window->priv->timer);
 
-        /* just in case they weren't removed */
         remove_key_events (window);
 
+        remove_command_watch (window);
+
+        /* If a dialog is up we need to signal it
+           and wait on it */
+        if (window->priv->pid > 0) {
+                signal_pid (window->priv->pid, SIGTERM);
+        }
+        gs_window_dialog_finish (window);
+
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -1199,6 +1348,7 @@ gs_window_new (GdkScreen *screen,
         GObject     *result;
 
         result = g_object_new (GS_TYPE_WINDOW,
+                               "type", GTK_WINDOW_POPUP,
                                "screen", screen,
                                "monitor", monitor,
                                "lock-enabled", lock_enabled,
